/*
 * Copyright (c) 1998 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

#include <threads/pthread_internal.h>

int
pthread_mutex_destroy(pthread_mutex_t *m)
{
	int	p;
	
	p = splpreempt();
	pthread_lock(&m->mlock);

	/*
	 * If the mutex can be locked, its safe to destroy it. However,
	 * we don't actually bother to do anything.
	 */
	if (m->holder == 0) {
		m->holder = NULL_THREADPTR;
		m->count  = 0;
		pthread_unlock(&m->mlock);
		splx(p);
		return 0;
	}
	pthread_unlock(&m->mlock);
	splx(p);
	return EBUSY;
}
