/*
 * Copyright (c) 1996, 1998 University of Utah and the Flux Group.
 * All rights reserved.
 * 
 * This file is part of the Flux OSKit.  The OSKit is free software, also known
 * as "open source;" you can redistribute it and/or modify it under the terms
 * of the GNU General Public License (GPL), version 2, as published by the Free
 * Software Foundation (FSF).  To explore alternate licensing terms, contact
 * the University of Utah at csl-dist@cs.utah.edu or +1-801-585-3271.
 * 
 * The OSKit is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GPL for more details.  You should have
 * received a copy of the GPL along with the OSKit; see the file COPYING.  If
 * not, write to the FSF, 59 Temple Place #330, Boston, MA 02111-1307, USA.
 */

/*
 * Set priority of a thread.
 */
#include <threads/pthread_internal.h>

int
pthread_setschedparam(pthread_t tid, int policy,
		      const struct sched_param *param)
{
	pthread_thread_t  *pthread;

	if (tid < 0 || tid >= THREADS_MAX_THREAD ||
	    tidtothread(tid) == NULL_THREADPTR)
		return EINVAL;

	pthread = tidtothread(tid);

#ifndef CPU_INHERIT
	switch (policy) {
	case SCHED_FIFO:
	case SCHED_RR:
		break;

	case SCHED_DECAY:
		panic("pthread_setschedparam: SCHED_DECAY not supported yet");
		break;

	default:
		return EINVAL;
	}
#endif
	/*
	 * Pass this off to the scheduler module. The simple POSIX
	 * model allows the policy and priority to be changed.
	 */
	pthread_sched_change_state(pthread, param->priority, policy);

	/*
	 * Check for reschedule since current thread might no longer be the
	 * highest priority thread.
	 */
	check_yield();
	
	return 0;
}

